import pynlo
import numpy as np
import matplotlib.pyplot as plt
plt.ion()
π = np.pi

def pulsator(FWHM=0.050,EPP=50e-12,GDD=0.0,TOD=0.0,window=10.0,points=2**13,pulseWL=1550):
	# pulseWL (nm) pulse central wavelength
	pulse = pynlo.light.DerivedPulses.GaussianPulse(
	# pulse = pynlo.light.DerivedPulses.SechPulse(
		power = 1, # Power will be scaled by set_epp
		T0_ps                = FWHM/1.76, # FWHM (ps) pulse duration
		center_wavelength_nm = pulseWL,   # 
		time_window_ps       = window,    # (ps)   simulation window
		GDD = GDD,                        # (ps^2) Group delay dispersion
		TOD = TOD,                        # (ps^3) Third order dispersion
		NPTS = points,                    #        simulation points
		frep_MHz = 100,
		# chirp2=10,
		# chirp3=10,
		power_is_avg = False
	)
	# pulse = pynlo.light.DerivedPulses.CWPulse(avg_power=1, center_wavelength_nm=pulseWL, time_window_ps=window, NPTS=Points, offset_from_center_THz=None)
	# pulse = pynlo.light.DerivedPulses.NoisePulse(          center_wavelength_nm=pulseWL, time_window_ps=window, NPTS=Points, frep_MHz=None)
	pulse.set_epp(EPP) # EPP = Energy per pulse (J)
	pulse.FWHM = FWHM # initial width
	# if not shape is None:
	# 	pulse = shaper(pulse,shape,shape_coverage,FWHM)
	return pulse

def fibrator(length=20,α=0,β2=-120,β3=0.00,β4=0.005,Γ=1000,fibWL=1550):
	# α  (dB/cm) attentuation coefficient 
	# β2 (ps^2/km)
	# β3 (ps^3/km)
	# β4 (ps^4/km)
	# Γ  (1/(W km)
	# length (mm)
	# fibWL  (nm) Center WL of fiber
	fiber = pynlo.media.fibers.fiber.FiberInstance()
	fiber.generate_fiber(
		length * 1e-3,
		center_wl_nm = fibWL,
		betas = (β2, β3, β4),
		gamma_W_m = Γ * 1e-3,
		gvd_units = 'ps^n/km',
		gain = -np.log((10**(α*0.1)))*100 # convert from dB/cm to 1/m
	)
	return fiber

def solve(pulse,fiber,steps=512):
	solver = pynlo.interactions.FourWaveMixing.SSFM.SSFM(
		local_error = 0.005,
		USE_SIMPLE_RAMAN = True,
		disable_Raman = False,
		disable_self_steepening = False
	)
	return solver.propagate(pulse_in = pulse, fiber = fiber, n_steps = steps)

# nearest interpolation (convert [0,1] to valid range)
# Note that 0 and 1 match the border pixels. an other convension could map [0,1[
# def sampler1d(texture,coord):
# 	texture = np.array(texture)
# 	coord=np.array(coord)
# 	print(coord)
# 	clipped = np.clip(coord,0,1)
# 	print(clipped)
# 	indices = np.round(clipped*(len(texture)-1)).astype(int)
# 	print(indices)
# 	return texture[indices]

# broken version
# def shaper(pulse,to_encode,ν_min,ν_max):
# 	to_encode = np.concatenate([[1],to_encode,[1]]) # identity outside the domain
# 	s = sampler1d(to_encode,(pulse.W_THz-ν_min)/(ν_max-ν_min))
# 	pulse.set_AW(pulse.AW*s)
# 	return pulse

def shaper(pulse,to_encode,ν_min,ν_max):
	N = len(to_encode)
	for (n,φ) in enumerate(to_encode):
		print(n/N)
	# to_encode = np.concatenate([[1],to_encode,[1]]) # identity outside the domain
	# s = sampler1d(to_encode,(pulse.W_THz-ν_min)/(ν_max-ν_min))
	# pulse.set_AW(pulse.AW*s)
	return pulse




AW = pulse.AW
λ = window(ω0,width)
AW = translate(λ*AW) + ((1-λ)*AW)
		let s = exp(-σ*((x-μ)**2))
		return s*l*cos(-x*φshift*100) + s*(1-l)
	}
	function fun_s_im(x,μ,σ,φaround,φwidth,φshift) {
		let l = g((x-φaround)/φwidth)
		let s = exp(-σ*((x-μ)**2))
		return s*l*sin(-x*φshift*100)
	}









from viz import vizabs

def debug_test():
	pulse = pulsator(FWHM=0.050,EPP=50e-12,GDD=0.0,TOD=0.0,window=10.0,points=2**13,pulseWL=1550)
	plt.figure();vizabs(pulse)
	W = pulse.W_THz[np.argmax(np.abs(pulse.AW))]
	pulse = shaper(pulse,np.zeros(100),W-100,W+100)
	plt.figure();vizabs(pulse)
	fiber = fibrator()
	y, AW, AT, pulse_out = solve(pulse,fiber,100)
	plt.figure();vizabs(pulse_out)

def physical(input):
	pulse = pulsator(FWHM=0.050,EPP=50e-12,GDD=0.0,TOD=0.0,window=10.0,points=2**13,pulseWL=1550)
	W = pulse.W_THz[np.argmax(np.abs(pulse.AW))]
	pulse = shaper(pulse,input,W-100,W+100)
	fiber = fibrator()
	y, AW, AT, pulse_out = solve(pulse,fiber)
	return pulse_out

def physical_dataset(input,sample_name):
	pulse = pulsator(FWHM=0.050,EPP=50e-12,GDD=0.0,TOD=0.0,window=10.0,points=2**13,pulseWL=1550)
	W = pulse.W_THz[np.argmax(np.abs(pulse.AW))]
	pulse = shaper(pulse,input,W-100,W+100)
	fiber = fibrator()
	y, AW, AT, pulse_out = solve(pulse,fiber)
	np.save(f"sample/phase_in_{sample_name}"        ,pulse.AW)
	np.save(f"sample/phase_in_energy_{sample_name}" ,pulse.calc_epp())
	np.save(f"sample/phase_out_{sample_name}"       ,pulse_out.AW)
	np.save(f"sample/phase_out_energy_{sample_name}",pulse_out.calc_epp())
	return pulse_out

for sample in ["test"]: #range(2**14):
	physical_dataset(np.exp(2*π*1j*np.random.rand(256)),sample)
	physical_dataset(np.random.rand(256)               ,sample)


def physical_debug(input):
	pulse = pulsator(FWHM=0.050,EPP=50e-12,GDD=0.0,TOD=0.0,window=10.0,points=2**13,pulseWL=1550)
	W = pulse.W_THz[np.argmax(np.abs(pulse.AW))]
	pulse = shaper(pulse,input,W-100,W+100)
	fiber = fibrator()
	y, AW, AT, pulse_out = solve(pulse,fiber)
	plt.figure()
	viz_pair(pulse,pulse_out)
	plt.figure()
	viz_pair_t(pulse,pulse_out)
	return pulse_out



# physical_dataset(np.exp(2*π*1j*np.random.rand(256)),sample)

physical_debug(np.ones(256))
physical_debug(np.exp(2*π*1j*np.random.rand(256)))

input = (0j+1)*np.ones(256)
input[0:64]=1j
physical_debug(input)


input[:]=1j
physical_debug(input)



physical_debug(np.exp(2*π*1j*np.random.rand(32)))



# 200 samples = 12 min
# 20 samples = 5 min
